//--------------------------------------------------------------
// File     : stm32_ub_rn42.c
// Datum    : 01.12.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO, UART, MISC
// Funktion : Bluetooth Modul (mit RN-42 Chip)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_rn42.h"
#include <string.h>


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_RN42_Init_Uart(void);
ErrorStatus P_RN42_CMD_Mode(void);
void P_RN42_SendByte(uint16_t wert);
void P_RN42_SendString(char *ptr, RN42_LASTBYTE_t last_byte);
void P_RN42_Receive(uint16_t wert);




//--------------------------------------------------------------
// Init vom RN42-Modul und aktivieren vom Command-Mode
// Return_wert :
//  -> ERROR   , wenn Modul nicht gefunden wurde
//  -> SUCCESS , wenn Modul OK und im Command-Mode
//--------------------------------------------------------------
ErrorStatus UB_RN42_Init(void)
{
  ErrorStatus ret_wert=ERROR; 

  // init vom UART
  P_RN42_Init_Uart();

  // RX-Puffer vorbereiten
  RN42_RX.rx_buffer[0]=RN42_RX_END_CHR;
  RN42_RX.wr_ptr=0;
  RN42_RX.status=RN42_RX_EMPTY;

  RN42.buffer[0]=0x00;

  // in CMD-Mode wechseln
  ret_wert=P_RN42_CMD_Mode();
  if(ret_wert==SUCCESS) {
    RN42.mode=RN42_MODE_CMD;
  }
  else {
    RN42.mode=RN42_MODE_ERROR;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// RN42-Modul vom Command-Mode in Data-Mode schalten
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , alles ok
//--------------------------------------------------------------
ErrorStatus UB_RN42_DATA_Mode(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout;

  if(RN42.mode!=RN42_MODE_CMD) return(ERROR);

  P_RN42_SendString("---",RN42_CR);

  timeout=0;
  do {
    timeout++;
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      timeout=RN42_RX_TIMEOUT;
      if(strncmp(RN42.buffer,"END",3)==0) {
        ret_wert=SUCCESS;
        RN42.mode=RN42_MODE_DATA;
      }
    }
  }
  while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));

  return(ret_wert);
}


//--------------------------------------------------------------
// RN42-Modul vom Data-Mode in Command-Mode schalten
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , alles ok
//--------------------------------------------------------------
ErrorStatus UB_RN42_CMD_Mode(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout;

  if(RN42.mode!=RN42_MODE_DATA) return(ERROR);

  P_RN42_SendString("$$$",RN42_NONE);

  timeout=0;
  do {
    timeout++;
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      timeout=RN42_RX_TIMEOUT;
      if(strncmp(RN42.buffer,"CMD",3)==0) {
        ret_wert=SUCCESS;
        RN42.mode=RN42_MODE_CMD;
      }
    }
  }
  while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));

  return(ret_wert);
}


//--------------------------------------------------------------
// einen String per RN42 ber Bluetooth empfangen (im Daten-Mode)
// (der Empfang wird per Interrupt abgehandelt)
// diese Funktion muss zyklisch gepollt werden
// Return Wert :
//  -> wenn nichts empfangen = RN42_RX_EMPTY
//  -> wenn String empfangen = RN42_RX_READY -> String steht in *ptr
//  -> wenn Puffer voll      = RN42_RX_FULL
//--------------------------------------------------------------
RN42_RXSTATUS_t UB_RN42_ReceiveString(char *ptr)
{
  RN42_RXSTATUS_t ret_wert=RN42_RX_EMPTY;
  uint8_t n,wert;

  if(RN42_RX.status==RN42_RX_READY) {
    ret_wert=RN42_RX_READY;
    // Puffer kopieren
    n=0;
    do {
      wert=RN42_RX.rx_buffer[n];
      if(wert!=RN42_RX_END_CHR) {
        ptr[n]=wert;
        n++;
      }
    }while(wert!=RN42_RX_END_CHR);
    // Stringendekennung
    ptr[n]=0x00;
    // RX-Puffer lschen
    RN42_RX.rx_buffer[0]=RN42_RX_END_CHR;
    RN42_RX.wr_ptr=0;
    RN42_RX.status=RN42_RX_EMPTY;
  }
  else if(RN42_RX.status==RN42_RX_FULL) {
    ret_wert=RN42_RX_FULL;
    // RX-Puffer lschen
    RN42_RX.rx_buffer[0]=RN42_RX_END_CHR;
    RN42_RX.wr_ptr=0;
    RN42_RX.status=RN42_RX_EMPTY;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// einen String per RN42 ber Bluetooth senden (im Daten-Mode)
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , alles ok
//--------------------------------------------------------------
ErrorStatus UB_RN42_SendString(char *ptr, RN42_LASTBYTE_t last_byte)
{
  ErrorStatus ret_wert=ERROR;

  if(RN42.mode!=RN42_MODE_DATA) return(ERROR);

  P_RN42_SendString(ptr,last_byte);

  ret_wert=SUCCESS;

  return(ret_wert);
}


//--------------------------------------------------------------
// Bluetooth-Adresse vom RN42-Modul auslesen (im Command-Mode)
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , alles ok -> Wert steht in "RN42.buffer"
//--------------------------------------------------------------
ErrorStatus UB_RN42_Get_BT_Adr(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout;

  if(RN42.mode!=RN42_MODE_CMD) return(ERROR);

  P_RN42_SendString("GB",RN42_CR);

  timeout=0;
  do {
    timeout++;
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      timeout=RN42_RX_TIMEOUT;
      if(strlen(RN42.buffer)==12) {
        ret_wert=SUCCESS;
      }
    }
  }
  while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));

  return(ret_wert);
}


//--------------------------------------------------------------
// Bluetooth-Adresse vom Ziel setzen (im Command-Mode)
// (Adresse muss 12 Zeichen lang sein)
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , alles ok
//--------------------------------------------------------------
ErrorStatus UB_RN42_Set_BT_Adr(char *ptr)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout, len;

  if(RN42.mode!=RN42_MODE_CMD) return(ERROR);

  len=strlen(ptr);
  if(len!=12) return(ERROR);

  strcpy(RN42.buffer,"SR,");
  strcat(RN42.buffer,ptr);

  P_RN42_SendString(RN42.buffer,RN42_CR);

  timeout=0;
  do {
    timeout++;
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      timeout=RN42_RX_TIMEOUT;
      if(strncmp(RN42.buffer,"AOK",3)==0) {
        ret_wert=SUCCESS;
      }
    }
  }
  while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));

  return(ret_wert);
}


//--------------------------------------------------------------
// Pin-Nummer vom RN42-Modul setzen (im Command-Mode)
// (Pin darf max 20 Zeichen lang sein)
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , alles ok
//--------------------------------------------------------------
ErrorStatus UB_RN42_Set_BT_Pin(char *ptr)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout, len;

  if(RN42.mode!=RN42_MODE_CMD) return(ERROR);

  len=strlen(ptr);
  if(len==0) return(ERROR);
  if(len>20) return(ERROR);

  strcpy(RN42.buffer,"SP,");
  strcat(RN42.buffer,ptr);

  P_RN42_SendString(RN42.buffer,RN42_CR);

  timeout=0;
  do {
    timeout++;
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      timeout=RN42_RX_TIMEOUT;
      if(strncmp(RN42.buffer,"AOK",3)==0) {
        ret_wert=SUCCESS;
      }
    }
  }
  while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));

  return(ret_wert);
}


//--------------------------------------------------------------
// Bluetooth-Name vom RN42-Modul setzen (im Command-Mode)
// (Name darf max 20 Zeichen lang sein)
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , alles ok
//--------------------------------------------------------------
ErrorStatus UB_RN42_Set_BT_Name(char *ptr)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout, len;

  if(RN42.mode!=RN42_MODE_CMD) return(ERROR);

  len=strlen(ptr);
  if(len==0) return(ERROR);
  if(len>20) return(ERROR);

  strcpy(RN42.buffer,"SN,");
  strcat(RN42.buffer,ptr);

  P_RN42_SendString(RN42.buffer,RN42_CR);

  timeout=0;
  do {
    timeout++;
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      timeout=RN42_RX_TIMEOUT;
      if(strncmp(RN42.buffer,"AOK",3)==0) {
        ret_wert=SUCCESS;
      }
    }
  }
  while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));

  return(ret_wert);
}


//--------------------------------------------------------------
// RN42-Modul mit der Ziel-Adresse verbinden (im Command-Mode)
// Return_wert :
//  -> ERROR   , Fehler (nicht verbunden) -> Command-Mode
//  -> SUCCESS , OK (verbunden) -> im Data-Mode
//--------------------------------------------------------------
ErrorStatus UB_RN42_Connect(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t ok;

  if(RN42.mode!=RN42_MODE_CMD) return(ERROR);

  P_RN42_SendString("C",RN42_CR);

  ok=0;
  do {
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      if(strncmp(RN42.buffer,"CONNECT failed",14)==0) {
        ret_wert=ERROR;
        ok=1;
      }
      else if(strncmp(RN42.buffer,"CONNECT",7)==0) {
        ret_wert=SUCCESS;
        RN42.mode=RN42_MODE_DATA;
        ok=2;
      }
    }
  }
  while(ok==0);

  return(ret_wert);
}


//--------------------------------------------------------------
// Verbindung vom RN42-Modul mit Ziel trennen (im Command-Mode)
// Return_wert :
//  -> ERROR   , Fehler
//  -> SUCCESS , OK (getrennt)
//--------------------------------------------------------------
ErrorStatus UB_RN42_Disconnect(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout;

  if(RN42.mode!=RN42_MODE_CMD) return(ERROR);

  P_RN42_SendString("K,",RN42_CR);

  timeout=0;
  do {
    timeout++;
    if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
      timeout=RN42_RX_TIMEOUT;
      if(strncmp(RN42.buffer,"KILL",4)==0) {
        ret_wert=SUCCESS;
      }
    }
  }
  while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// init vom UART
//--------------------------------------------------------------
void P_RN42_Init_Uart(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  USART_InitTypeDef USART_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  // Clock enable der TX und RX Pins
  RCC_AHB1PeriphClockCmd(RN42_TX_CLOCK, ENABLE);
  RCC_AHB1PeriphClockCmd(RN42_RX_CLOCK, ENABLE);

  // Clock enable der UART
  RCC_APB1PeriphClockCmd(RN42_UART_CLOCK, ENABLE);

  // UART Alternative-Funktions mit den IO-Pins verbinden
  GPIO_PinAFConfig(RN42_TX_PORT,RN42_TX_SOURCE,RN42_UART_AF);
  GPIO_PinAFConfig(RN42_RX_PORT,RN42_RX_SOURCE,RN42_UART_AF);

  // UART als Alternative-Funktion mit PushPull
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // TX-Pin
  GPIO_InitStructure.GPIO_Pin = RN42_TX_PIN;
  GPIO_Init(RN42_TX_PORT, &GPIO_InitStructure);
  // RX-Pin
  GPIO_InitStructure.GPIO_Pin =  RN42_RX_PIN;
  GPIO_Init(RN42_RX_PORT, &GPIO_InitStructure);

  // Oversampling
  USART_OverSampling8Cmd(RN42_UART_COM, ENABLE);

  // init mit Baudrate, 8Databits, 1Stopbit, keine Paritt, kein RTS+CTS
  USART_InitStructure.USART_BaudRate = RN42_UART_BAUD;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  USART_Init(RN42_UART_COM, &USART_InitStructure);

  // UART enable
  USART_Cmd(RN42_UART_COM, ENABLE);

  // RX-Interrupt enable
  USART_ITConfig(RN42_UART_COM, USART_IT_RXNE, ENABLE);

  // enable UART Interrupt-Vector
  NVIC_InitStructure.NVIC_IRQChannel = RN42_UART_INT;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}



//--------------------------------------------------------------
// interne Funktion
// versuchen das RN42-Modul in CMD-Mode zu schalten
//--------------------------------------------------------------
ErrorStatus P_RN42_CMD_Mode(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t timeout,cnt=0;

  // versuche in Command-Mode zu wechseln
  do {
    for(timeout=0;timeout<RN42_INIT_DELAY;timeout++) {
      // delay
    }

    P_RN42_SendString("$$$",RN42_NONE);

    timeout=0;
    do {
      timeout++;
      if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
        timeout=RN42_RX_TIMEOUT;
        if(strncmp(RN42.buffer,"CMD",3)==0) {
          ret_wert=SUCCESS;
        }
      }
    }
    while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));
    cnt++;
  }
  while((cnt<3) && (ret_wert==ERROR));

  if(ret_wert==ERROR) {
    // test ob Command-Mode schon aktiv
    for(timeout=0;timeout<RN42_INIT_DELAY;timeout++) {
      // delay
    }

    P_RN42_SendString("$$$",RN42_CR);

    timeout=0;
    do {
      timeout++;
      if(UB_RN42_ReceiveString(RN42.buffer)==RN42_RX_READY) {
        timeout=RN42_RX_TIMEOUT;
        if(strncmp(RN42.buffer,"?",RN42_CR)==0) {
          ret_wert=SUCCESS;
        }
      }
    }
    while((timeout<RN42_RX_TIMEOUT) && (ret_wert==ERROR));
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// ein Byte per UART senden
//--------------------------------------------------------------
void P_RN42_SendByte(uint16_t wert)
{
  // warten bis altes Byte gesendet wurde
  while (USART_GetFlagStatus(RN42_UART_COM, USART_FLAG_TXE) == RESET);
  USART_SendData(RN42_UART_COM, wert);
}


//--------------------------------------------------------------
// interne Funktion
// einen String per UART senden
//--------------------------------------------------------------
void P_RN42_SendString(char *ptr, RN42_LASTBYTE_t last_byte)
{
  // sende kompletten String
  while (*ptr != 0) {
    P_RN42_SendByte(*ptr);
    ptr++;
  }
  // eventuell Endekennung senden
  if(last_byte==RN42_LFCR) {
    P_RN42_SendByte(0x0A); // LineFeed senden
    P_RN42_SendByte(0x0D); // CarrigeReturn senden
  }
  else if(last_byte==RN42_CRLF) {
    P_RN42_SendByte(0x0D); // CarrigeReturn senden
	P_RN42_SendByte(0x0A); // LineFeed senden
  }
  else if(last_byte==RN42_LF) {
    P_RN42_SendByte(0x0A); // LineFeed senden
  }
  else if(last_byte==RN42_CR) {
    P_RN42_SendByte(0x0D); // CarrigeReturn senden
  }
}



//--------------------------------------------------------------
// interne Funktion
// speichern des empfangenen Zeichens im Puffer
//--------------------------------------------------------------
void P_RN42_Receive(uint16_t wert)
{
  if(RN42_RX.wr_ptr<RN42_RX_BUF_SIZE) {
    // wenn noch Platz im Puffer
    if(RN42_RX.status==RN42_RX_EMPTY) {
      // wenn noch keine Endekennung empfangen wurde
      if((wert>=RN42_RX_FIRST_CHR) && (wert<=RN42_RX_LAST_CHR)) {
        // Byte im Puffer speichern
        RN42_RX.rx_buffer[RN42_RX.wr_ptr]=wert;
        RN42_RX.wr_ptr++;
      }
      if(wert==RN42_RX_END_CHR) {
        // wenn Endekennung empfangen
        RN42_RX.rx_buffer[RN42_RX.wr_ptr]=wert;
        RN42_RX.status=RN42_RX_READY;
      }
    }
  }
  else {
    // wenn Puffer voll ist
    RN42_RX.status=RN42_RX_FULL;
  }
}



//--------------------------------------------------------------
// UART-Interrupt
//--------------------------------------------------------------
void RN42_UART_IRQ(void) {
  uint16_t wert;

  if (USART_GetITStatus(RN42_UART_COM, USART_IT_RXNE) == SET) {
    // wenn ein Byte im Empfangspuffer steht
    wert=USART_ReceiveData(RN42_UART_COM);
    // Byte speichern
    P_RN42_Receive(wert);
  }
}

